USING SFDC.*.
USING Progress.Json.ObjectModel.*.
USING System.Net.*.
USING System.IO.*.

{SFDC/SFDCTT.i}
   
DEFINE VARIABLE cSFDCBulk     AS SFDCBulkLib NO-UNDO. 
DEFINE VARIABLE httCustomer AS HANDLE       NO-UNDO. 
DEFINE VARIABLE AccountJobId   AS CHARACTER    NO-UNDO. 
DEFINE VARIABLE AccountBatchId AS CHARACTER    NO-UNDO.
DEFINE VARIABLE AccountStatus  AS CHARACTER    NO-UNDO.
DEFINE VARIABLE httContact AS HANDLE       NO-UNDO. 
DEFINE VARIABLE ContactJobId   AS CHARACTER    NO-UNDO. 
DEFINE VARIABLE ContactBatchId AS CHARACTER    NO-UNDO.
DEFINE VARIABLE ContactStatus  AS CHARACTER    NO-UNDO.
DEFINE VARIABLE vi             AS INTEGER      NO-UNDO.
DEFINE VARIABLE lcMessage      AS LONGCHAR     NO-UNDO.

cSFDCBulk = NEW SFDC.SFDCBulkLib ().
cSFDCBulk:cSFDC:loginToSFDC(FALSE).
 
httCustomer = TEMP-TABLE ttCustomer:HANDLE.
httContact = TEMP-TABLE ttContact:HANDLE.
 
/* create two async jobs - one to get all accounts and one to get all contacts */
AccountJobId = cSFDCBulk:CreateJob("query", "Account").
ContactJobId = cSFDCBulk:CreateJob("query", "Contact").

/* create a batch for each  */
AccountBatchId = cSFDCBulk:CreateBatch("Account", httCustomer, AccountJobId).
ContactBatchId = cSFDCBulk:CreateBatch("Contact", httContact, ContactJobId).

ContactStatus = "Initiated in SFDC".
AccountStatus = "Initiated in SFDC".

/* check the status of each batch and retreive each batch when completed */
processLoop:
REPEAT:
  /* you must pause!! If you do not, SFDC will not accept your requests */
  System.Threading.Thread:Sleep(1000).
  
     
  /* check the Contact job */
  IF ContactStatus <> "JobFullyProcessed" THEN
  DO:
    ContactStatus = cSFDCBulk:CheckBatchStatus(ContactJobId, ContactbatchId). 
    
    IF ContactStatus = "failed" OR ContactStatus = "InvalidBatch" THEN  
    DO:
      MESSAGE "Contact Batch failed with Status:  "  ContactStatus
        VIEW-AS ALERT-BOX.
      RETURN.
    END.
    
    IF ContactStatus = "Completed" THEN 
    DO:
      cSFDCBulk:retrieveBatch(httContact, ContactJobId, ContactbatchId).
      ContactStatus = "JobFullyProcessed".
    END.
  END.   
     
  /* check the Account job */
  IF AccountStatus <> "JobFullyProcessed" THEN
  DO:
    AccountStatus = cSFDCBulk:CheckBatchStatus(AccountJobId, AccountbatchId). 
    
    IF AccountStatus = "failed" OR AccountStatus = "InvalidBatch" THEN  
    DO:
      MESSAGE "Account Batch failed with Status:  "  AccountStatus
        VIEW-AS ALERT-BOX.
      RETURN.
    END.
    
    IF AccountStatus = "Completed" THEN 
    DO:
      cSFDCBulk:retrieveBatch(httCustomer,AccountJobId,AccountbatchId).
      AccountStatus = "JobFullyProcessed".
    END.
  END.   
  
  /* infinit loop protection */
  vi = vi + 1.
  IF vi  > 10000 THEN 
  DO:
    MESSAGE "You waited how long to see this message??"
      VIEW-AS ALERT-BOX.
    RETURN.
  END. 
  
  IF AccountStatus = "JobFullyProcessed" AND ContactStatus = "JobFullyProcessed" THEN 
    LEAVE processloop.
    
  MESSAGE 
    "Looping" SKIP 
    "Account Status = " AccountStatus SKIP 
    "Contact Status = " ContactStatus
    VIEW-AS ALERT-BOX.
END.

cSFDCBulk:CloseJob(AccountJobId).
cSFDCBulk:CloseJob(ContactJobId).


/* display the results for the demo */  
httCustomer:WRITE-JSON ("LongChar", lcMessage, TRUE).
Util.DemoUtils:BigMessage(lcMessage).

/* ------------------------ */
CATCH e AS Progress.Lang.Error:
  DEFINE VARIABLE oWebException AS System.Net.WebException NO-UNDO.
  DEFINE VARIABLE Response      AS HttpWebResponse         NO-UNDO.
  DEFINE VARIABLE SR            AS StreamReader            NO-UNDO.
  DEFINE VARIABLE lcResponse    AS LONGCHAR                NO-UNDO.
  
  /* useful messages returned by the DotNet WebClient object are burried a little */
  IF  TYPE-OF(e,System.Net.WebException) THEN 
  DO:
    oWebException = CAST (e, System.Net.WebException). 
    Response = CAST(oWebException:Response, HttpWebResponse).
    SR = NEW StreamReader(Response:GetResponseStream(),System.Text.Encoding:UTF8).
    lcResponse = SR:ReadToEnd().

    MESSAGE STRING (lcResponse) SKIP Response:StatusCode:value__
      VIEW-AS ALERT-BOX.   
  END.
  ELSE 
    MESSAGE 
      e:GetMessage (1)
      VIEW-AS ALERT-BOX.
END. 

