/*------------------------------------------------------------------------
   File        : SFDCBulkAPI.p
   Author(s)   : Gerry Winning
                 gwinning@gmail.com
 ---------------------------------------------------------------------- */

ROUTINE-LEVEL ON ERROR UNDO, THROW. 

USING System.Net.*.
USING System.Text.*.
 
DEFINE VARIABLE hParser      AS HANDLE    NO-UNDO.
DEFINE VARIABLE lcResult     AS LONGCHAR  NO-UNDO.
DEFINE VARIABLE vcMode       AS CHARACTER NO-UNDO.
DEFINE VARIABLE fldData      AS CHARACTER NO-UNDO.
DEFINE VARIABLE Id           AS CHARACTER NO-UNDO.
DEFINE VARIABLE JobState     AS CHARACTER NO-UNDO. 
DEFINE VARIABLE stateMessage AS CHARACTER NO-UNDO.
DEFINE VARIABLE resultId     AS CHARACTER NO-UNDO.
  
/* **********************   Internal Procedures  *********************** */

PROCEDURE checkBatchStatus:
  /*------------------------------------------------------------------------------
      Purpose:                                      
      Notes:                                      
  ------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER pServerURL AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER pSessionId AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER JobId AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER batchId AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER vState AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER pError AS CHARACTER NO-UNDO.
  
  DEFINE VARIABLE oWebClient AS WebClient NO-UNDO.
  DEFINE VARIABLE vURL       AS CHARACTER NO-UNDO.
  
  vURL = pServerURL + "/services/async/24.0/job/" + JobId + "/batch/" + batchId.
  
  /* The COM http client would read the cache - make the URL unique */
  vURL = vURL + "?antiCache=" + STRING (NOW).  
  
  oWebClient = NEW WebClient().
  oWebClient:Encoding = Encoding:UTF8.
  oWebClient:Headers:Set("X-SFDC-Session", pSessionId).
  lcResult = oWebClient:DownloadString (vURL). 
  
  Util.DemoUtils:BigXMLMessage(lcResult). 
  
  jobState = "".
  vcMode = "checkBatch".
  RUN parseSaxResult.

  IF jobState = "Failed" THEN
  DO: 
    pError = stateMessage.
    RETURN.
  END.
  vState = jobState.
  RETURN.
  
END PROCEDURE.

PROCEDURE CloseJob:
  /*------------------------------------------------------------------------------
      Purpose:                                      
      Notes:                                      
  ------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER pServerURL AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER pSessionId AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER JobId AS CHARACTER NO-UNDO.

  DEFINE VARIABLE vURL       AS CHARACTER NO-UNDO.
  DEFINE VARIABLE xmlRequest AS LONGCHAR  NO-UNDO.
  DEFINE VARIABLE oWebClient AS WebClient NO-UNDO.
    
  xmlRequest = '<?xml version="1.0" encoding="UTF-8"?>' +
    '<jobInfo xmlns="http://www.force.com/2009/06/asyncapi/dataload">' +
    '<state>Closed</state>' +
    '</jobInfo>'.
  vURL = pServerURL + "/services/async/24.0/job/" + JobId.
  oWebClient = NEW WebClient().
  oWebClient:Encoding = Encoding:UTF8.  
  oWebClient:Headers:Set("Content-Type", "application/xml; charset=UTF-8").
  oWebClient:Headers:Set("X-SFDC-Session", pSessionId).
  lcResult = oWebClient:UploadString(vURL, "Post", xmlRequest).
  
  RETURN.
 
END PROCEDURE.

PROCEDURE CreateBatch:
  /*------------------------------------------------------------------------------
      Purpose:                                      
      Notes:                                      
  ------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER pServerURL AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER pSessionId AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER pQuery AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER pJobId AS CHARACTER NO-UNDO. 
  DEFINE INPUT PARAMETER pContentType AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER BatchId AS CHARACTER NO-UNDO.
  
  DEFINE VARIABLE vURL       AS CHARACTER NO-UNDO.
  DEFINE VARIABLE oWebClient AS WebClient NO-UNDO.
  
  vURL = pServerURL + "/services/async/24.0/job/" + pJobId + "/batch".
  oWebClient = NEW WebClient().
  oWebClient:Encoding = Encoding:UTF8. 
  oWebClient:Headers:Set("Content-Type", "application/xml; charset=UTF-8").
  oWebClient:Headers:Set("X-SFDC-Session", pSessionId).
  lcResult = oWebClient:UploadString(vURL, "Post", pQuery).
  
  Util.DemoUtils:BigXMLMessage(lcResult).

  vcMode = "CreateBatch".
  RUN parseSaxResult.
  BatchId = Id.

  RETURN.

END PROCEDURE.

PROCEDURE CreateJob:
  /*------------------------------------------------------------------------------
      Purpose:                                      
      Notes:                                      
  ------------------------------------------------------------------------------*/

  DEFINE INPUT  PARAMETER pServerURL AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER pSessionId AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER pOperation AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER pObject AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER pConcurMode AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER pContentType AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER JobId AS CHARACTER NO-UNDO.
  
  DEFINE VARIABLE xmlRequest AS LONGCHAR  NO-UNDO.
  DEFINE VARIABLE oWebClient AS WebClient NO-UNDO.
  DEFINE VARIABLE vURL       AS CHARACTER NO-UNDO.

  xmlRequest = '<?xml version="1.0" encoding="UTF-8"?> ' +
    '<jobInfo xmlns="http://www.force.com/2009/06/asyncapi/dataload"> ' +
    '<operation>' + pOperation + '</operation> ' +
    '<object>' + pObject + '</object> ' +
    '<concurrencyMode>' + pConcurMode + '</concurrencyMode> ' +
    '<contentType>' + pContentType + '</contentType> ' +
    '</jobInfo>'.
     
  Util.DemoUtils:BigXMLMessage(xmlRequest).

  /* rest-like URL */
  vURL = pServerURL + "/services/async/24.0/job". 
  oWebClient = NEW WebClient().
  oWebClient:Encoding = Encoding:UTF8.  
  oWebClient:Headers:Set("Content-Type", "application/xml; charset=UTF-8").
  oWebClient:Headers:Set("X-SFDC-Session", pSessionId).
  lcResult = oWebClient:UploadString(vURL, "Post", xmlRequest).

  Util.DemoUtils:BigXMLMessage(lcResult).

  vcMode = "CreateJob".
  RUN parseSaxResult.
  jobId = id.
  RETURN. 
END PROCEDURE.

PROCEDURE characters:
  /*------------------------------------------------------------------------------
    Purpose:     
    Parameters:  <none>
    Notes:       
  ------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER charData AS MEMPTR.
  DEFINE INPUT PARAMETER numChars AS INTEGER.
  
  /* With the SAX parser, there's no guarentee that all the data will come 
       through in one call to characters. We need to store the data somewhere
       so that we can accumulate the string. 
         
       The actual assignment of the values is being done in endElement.   */   
  fldData = fldData + GET-STRING(charData,1).
END PROCEDURE.

PROCEDURE endElement:
  DEFINE INPUT PARAMETER namespaceURI AS CHARACTER.
  DEFINE INPUT PARAMETER localName    AS CHARACTER.
  DEFINE INPUT PARAMETER qname        AS CHARACTER.
  
  DEFINE VARIABLE viExtent AS INTEGER NO-UNDO.
  
  fldData = TRIM(fldData).
  
  CASE vcMode:
    WHEN "CreateJob"  THEN 
      DO:
        CASE localName:
          WHEN "id" THEN
            Id = fldData. 
        END CASE.
      END.
    WHEN "CreateBatch" THEN
      DO:
        CASE localName:
          WHEN "id" THEN
            Id = fldData. 
        END CASE.
      END.
    WHEN "checkBatch" THEN
      DO:
        CASE localName:
          WHEN "id" THEN
            id = fldData.
          WHEN "state" THEN 
            jobState = fldData.
          WHEN "stateMessage" THEN
            stateMessage = fldData.
        END CASE.
      END.
    WHEN "getResultId" THEN
      IF localName = "result" THEN
        resultId = fldData.
  END CASE.
  fldData = "".
  RETURN.
   
END PROCEDURE.

PROCEDURE parseSaxResult:
  /*------------------------------------------------------------------------------
      Purpose:                                                                        
      Notes:                                                                      
  ------------------------------------------------------------------------------*/
    
  IF LENGTH (lcREsult) = 0 THEN   
    RETURN.
    
  CREATE SAX-READER hParser.
  /* the callbacks are in this same file */
  hParser:HANDLER = THIS-PROCEDURE.
  hParser:SET-INPUT-SOURCE("longchar", lcResult).
  hParser:SAX-PARSE().

  FINALLY:
    DELETE OBJECT hParser NO-ERROR.
  END.

END PROCEDURE.

PROCEDURE retrieveBatch:
  /*------------------------------------------------------------------------------
      Purpose:                                      
      Notes:                                      
  ------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER pServerURL AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER pSessionId AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER JobId AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER batchId AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER pResult AS LONGCHAR NO-UNDO.
  
  DEFINE VARIABLE oWebClient AS WebClient NO-UNDO. 
  DEFINE VARIABLE vURL       AS CHARACTER NO-UNDO.
   
  vURL = pServerURL + "/services/async/24.0/job/" + JobId + "/batch/" + batchId + "/result/".    
  oWebClient = NEW WebClient().
  oWebClient:Encoding = Encoding:UTF8.
  oWebClient:Headers:Set("X-SFDC-Session", pSessionId).
  lcResult = oWebClient:DownloadString (vURL). 
  
  Util.DemoUtils:BigXMLMessage(lcResult).
  
  vcMode = "getResultId".
  RUN parseSaxResult.  

  vURL = pServerURL + "/services/async/24.0/job/" + JobId + "/batch/" + batchId + "/result/" + resultId. 
  oWebClient = NEW WebClient().
  oWebClient:Encoding = Encoding:UTF8.
  oWebClient:Headers:Set("X-SFDC-Session", pSessionId).
  pResult = oWebClient:DownloadString (vURL). 
  
  Util.DemoUtils:BigXMLMessage(pResult).

  RETURN.    

END PROCEDURE.

PROCEDURE startElement :
  /*------------------------------------------------------------------------------
    Purpose:     
    Parameters:  <none>
    Notes:       
  ------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER namespaceURI AS CHARACTER.
  DEFINE INPUT PARAMETER localName    AS CHARACTER.
  DEFINE INPUT PARAMETER qName        AS CHARACTER.
  DEFINE INPUT PARAMETER attributes   AS HANDLE.
  

END PROCEDURE.