/*------------------------------------------------------------------------
   File        : SFDCBulkLib.cls
   Author(s)   : Gerry Winning
                 gwinning@gmail.com
 ---------------------------------------------------------------------- */
 
ROUTINE-LEVEL ON ERROR UNDO, THROW.   
USING Progress.Lang.*.
USING SFDC.*.
   
CLASS SFDC.SFDCBulkLib: 
  DEFINE VARIABLE hSFDCBulkAPI AS HANDLE NO-UNDO.
  
  DEFINE PUBLIC PROPERTY contentType AS CHARACTER NO-UNDO 
    GET.
    SET. 

  DEFINE PUBLIC PROPERTY CSFDC AS SFDCLib NO-UNDO 
    GET.
    SET.  

  /*------------------------------------------------------------------------------
      Purpose:                                      
      Notes:                                      
  ------------------------------------------------------------------------------*/

  CONSTRUCTOR PUBLIC SFDCBulkLib ( ):
    SUPER ().
    cSFDC = NEW SFDCLib().
    RUN SFDC/SFDCBulkAPI.p PERSISTENT SET hSFDCBulkAPI.  
    
    /* content type can be XML or CSV  (no JSON) */
    contentType = "XML".
  END CONSTRUCTOR.

  /*------------------------------------------------------------------------------
      Purpose:                                      
      Notes:                                      
  ------------------------------------------------------------------------------*/

  METHOD PUBLIC CHARACTER buildosql( htt AS HANDLE, SFDCTable AS CHARACTER ):
    DEFINE VARIABLE result AS CHARACTER NO-UNDO.
    
    RESULT = 'Select ' + cSFDC:buildFieldList(htt) + ' FROM ' + SFDCTable.
    RETURN result.
  END METHOD.
     
  /*------------------------------------------------------------------------------
      Purpose:                                      
      Notes:                                      
  ------------------------------------------------------------------------------*/
 
  METHOD PUBLIC CHARACTER CheckBatchStatus( pJobId AS CHARACTER, pBatchId AS CHARACTER ):
    DEFINE VARIABLE vError  AS CHARACTER NO-UNDO.
    DEFINE VARIABLE vStatus AS CHARACTER NO-UNDO. 
    
    RUN checkBatchStatus IN hSFDCBulkAPI (
      INPUT cSFDC:instanceURL,
      INPUT cSFDC:sessionId,
      INPUT pJobId,
      INPUT pBatchId,
      OUTPUT vStatus,
      OUTPUT vError ). 
  
    IF vError <> "" THEN 
      CSFDC:errorOut (vError).
    RETURN vStatus.
  END METHOD.

  /*------------------------------------------------------------------------------
      Purpose:                                      
      Notes:                                      
  ------------------------------------------------------------------------------*/

  METHOD PUBLIC VOID CloseJob( pJobIds AS CHARACTER  ):
    DEFINE VARIABLE vi AS INTEGER NO-UNDO.
    DO vi = 1 TO NUM-ENTRIES (pJobIds):
      RUN CloseJob IN hSFDCBulkAPI (
        INPUT cSFDC:instanceURL,
        INPUT cSFDC:sessionId,
        INPUT ENTRY (vi,pJobIds)). 
    END.
    RETURN. 
  END METHOD.

  /*------------------------------------------------------------------------------
      Purpose:                                      
      Notes:                                      
  ------------------------------------------------------------------------------*/

  METHOD PUBLIC CHARACTER CreateBatch( SFDCTable AS CHARACTER, hTT AS HANDLE, pJobId AS CHARACTER    ):
    DEFINE VARIABLE batchId AS CHARACTER NO-UNDO.
    DEFINE VARIABLE vosql   AS CHARACTER NO-UNDO.
    
    vosql = buildosql(hTT, SFDCTable).
     
    Util.DemoUtils:BigMessage(vosql). 
     
    RUN CreateBatch IN hSFDCBulkAPI(INPUT cSFDC:instanceURL,
      INPUT cSFDC:sessionId,
      INPUT vosql,
      INPUT pJobId,
      INPUT contentType,
      OUTPUT batchId).
      
    RETURN batchId.
  END METHOD.

  METHOD PUBLIC CHARACTER CreateJob(pOperation AS CHARACTER, pObject AS CHARACTER  ):
    DEFINE VARIABLE JobId AS CHARACTER NO-UNDO.
    RUN CreateJob IN hSFDCBulkAPI (
      INPUT cSFDC:instanceURL,
      INPUT cSFDC:sessionId,
      INPUT pOperation,
      INPUT pObject,
      INPUT "Parallel",
      INPUT contentType,
      OUTPUT jobId).
      
    RETURN jobId.
  END METHOD.

  /*------------------------------------------------------------------------------
      Purpose:                                      
      Notes:                                      
  ------------------------------------------------------------------------------*/

  METHOD PUBLIC VOID retrieveBatch( hTT AS HANDLE, pJobId AS CHARACTER, pBatchId AS CHARACTER  ):
    DEFINE VARIABLE lcResult       AS LONGCHAR  NO-UNDO.
    DEFINE VARIABLE vc             AS CHARACTER NO-UNDO.
    DEFINE VARIABLE vSerializeName AS CHARACTER NO-UNDO.

    RUN retrieveBatch IN hSFDCBulkAPI (
      INPUT cSFDC:instanceURL,
      INPUT cSFDC:sessionId,
      INPUT pJobId,
      INPUT pBatchId,
      OUTPUT lcResult). 
 
    /* the xml returns nodes named "records" instead of "Account".  by changing the "serialize-name" of the
       temp-table to "records", we can then use the Progress "read xml" method to read it in */
    vSerializeName = hTT:SERIALIZE-NAME .
    hTT:SERIALIZE-NAME = "Records".    
    hTT:READ-XML ("LongChar", lcResult, ?, ?, ?) NO-ERROR  .
    IF ERROR-STATUS:NUM-MESSAGES > 0 THEN 
    DO:
      IF ERROR-STATUS:GET-NUMBER (1) <> 13036 THEN  /* empty xml is OK - needs more QA */
      DO:
        hTT:SERIALIZE-NAME = vSerializeName.
        CSFDC:ErrorOut("Error reading xml from retrieveBatch - " + ERROR-STATUS:GET-MESSAGE (1)).
        RETURN.
      END.
    END.
    hTT:SERIALIZE-NAME = vSerializeName.
    
    RETURN.
  END METHOD.

END CLASS.