/*------------------------------------------------------------------------
   File        : SFDCLib.cls
   Author(s)   : Gerry Winning
                 gwinning@gmail.com
 ---------------------------------------------------------------------- */
ROUTINE-LEVEL ON ERROR UNDO, THROW.

USING Util.*.   
USING Progress.Lang.*. 
USING System.Net.*.
USING Progress.Json.ObjectModel.*.
USING System.XML.*.
USING System.Collections.Specialized.*.
USING System.IO.*.
USING System.Text.*.

CLASS SFDC.SFDCLib: 

  {SFDC/SFDCTT.i}

  DEFINE PUBLIC PROPERTY accessToken AS CHARACTER NO-UNDO 
    GET.
    SET.      
    
  DEFINE PUBLIC PROPERTY hSFDCSOAPAPI AS HANDLE NO-UNDO 
    GET.
    SET.    
    
  DEFINE PUBLIC PROPERTY hTTAccountTeamMember AS HANDLE NO-UNDO 
    GET(): 
      RETURN TEMP-TABLE ttAccountTeamMember:HANDLE.
    END GET.
    SET. 

  DEFINE PUBLIC PROPERTY hTTContact AS HANDLE NO-UNDO 
    GET():
      RETURN TEMP-TABLE ttContact:HANDLE.
    END GET.
    SET. 
    
  DEFINE PUBLIC PROPERTY hTTCustomer AS HANDLE NO-UNDO 
    GET():
      RETURN TEMP-TABLE ttCustomer:HANDLE.
    END GET.
    SET. 
    
  DEFINE PUBLIC PROPERTY hTTFieldsToNull AS HANDLE NO-UNDO 
    GET():
      RETURN TEMP-TABLE ttFieldsToNull:HANDLE.
    END GET.
    SET.  

  DEFINE PUBLIC PROPERTY hTTOpportunity AS HANDLE NO-UNDO 
    GET():
      RETURN TEMP-TABLE ttOpportunity:HANDLE.   
    END.
    SET. 
    
  DEFINE PUBLIC PROPERTY hTTOpportunityLineItem AS HANDLE NO-UNDO 
    GET():
      RETURN TEMP-TABLE ttOpportunityLineItem:HANDLE.   
    END.
    SET. 
       
  DEFINE PUBLIC PROPERTY hTTPricebookEntry AS HANDLE NO-UNDO 
    GET():
      RETURN TEMP-TABLE ttPricebookEntry:HANDLE.
    END GET.
    SET.  

  DEFINE PUBLIC PROPERTY hTTProduct AS HANDLE NO-UNDO 
    GET():
      RETURN TEMP-TABLE ttProduct:HANDLE.
    END GET.
    SET.  

  DEFINE PUBLIC PROPERTY hTTUser AS HANDLE NO-UNDO 
    GET():
      RETURN TEMP-TABLE ttUser:HANDLE.
    END GET.
    SET. 
     
  DEFINE PUBLIC PROPERTY hTTId AS HANDLE NO-UNDO 
    GET():
      RETURN TEMP-TABLE ttId:HANDLE.
    END GET.
    SET. 

  DEFINE PUBLIC PROPERTY instanceURL AS CHARACTER NO-UNDO 
    GET.
    SET.
     
  DEFINE PUBLIC PROPERTY SessionId AS CHARACTER NO-UNDO 
    GET.
    SET. 

  DEFINE PUBLIC PROPERTY SoapEndPoint AS CHARACTER NO-UNDO 
    GET.
    SET. 

  /*------------------------------------------------------------------------------
   Purpose:
   Notes:
  ------------------------------------------------------------------------------*/
		
  CONSTRUCTOR PUBLIC SFDCLib (  ):
    SUPER ().
    RUN SFDC/SFDCSOAPAPI.p PERSISTENT SET hSFDCSOAPAPI.      
    RETURN.  
  END CONSTRUCTOR.

  /*------------------------------------------------------------------------------
      Purpose:                                      
      Notes:                                      
  ------------------------------------------------------------------------------*/

  METHOD PUBLIC FINAL VOID addXmlsnsAttributes( hSaxWriter AS HANDLE ): 
    hSaxWriter:INSERT-ATTRIBUTE("xmlns:sfdc","urn:enterprise.soap.sforce.com").
    hSaxWriter:INSERT-ATTRIBUTE("xmlns:urn", "urn:enterprise.soap.sforce.com").
    hSaxWriter:INSERT-ATTRIBUTE("xmlns:urn1","urn:sobject.enterprise.soap.sforce.com"). 
    hSaxWriter:INSERT-ATTRIBUTE("xmlns:xsi","http://www.w3.org/2001/XMLSchema-instance").
    RETURN.
  END METHOD.

  /*------------------------------------------------------------------------------
      Purpose:                                                                        
      Notes:                                                                      
  ------------------------------------------------------------------------------*/

  METHOD PUBLIC CHARACTER buildFieldList( htt AS HANDLE ):
    DEFINE VARIABLE vsName AS CHARACTER NO-UNDO.
    DEFINE VARIABLE vi     AS INTEGER   NO-UNDO.
    DEFINE VARIABLE result AS CHARACTER NO-UNDO.
        
    loopy:
    DO vi = 1 TO hTT:DEFAULT-BUFFER-HANDLE:NUM-FIELDS:
      vsname = hTT:DEFAULT-BUFFER-HANDLE:BUFFER-FIELD (vi):SERIALIZE-NAME.
      IF vsname = "skipSFDC" THEN NEXT loopy.
      RESULT = 
        RESULT + 
        (IF RESULT = "" THEN "" ELSE ", ") + 
        vsname.
    END.
    
    RETURN result.

  END METHOD.
  
  /*------------------------------------------------------------------------------
      Purpose:                                                                        
      Notes:                                                                      
  ------------------------------------------------------------------------------*/
 
  METHOD PUBLIC VOID copyToTT( hTT AS HANDLE, pId AS CHARACTER):
    DEFINE VARIABLE vURL       AS CHARACTER         NO-UNDO.
    DEFINE VARIABLE oWebClient AS WebClient         NO-UNDO.
    DEFINE VARIABLE oParse     AS ObjectModelParser NO-UNDO.
    DEFINE VARIABLE oJSON      AS JsonObject        NO-UNDO.
    DEFINE VARIABLE vi         AS INTEGER           NO-UNDO.
    DEFINE VARIABLE hBuff      AS HANDLE            NO-UNDO.
    DEFINE VARIABLE lcResult   AS LONGCHAR          NO-UNDO.

    vURL = instanceURL + "/services/data/v24.0/sobjects/" + hTT:SERIALIZE-NAME + "/" + pId .
                
    oWebClient = NEW WebClient().
    oWebClient:Encoding = Encoding:UTF8.
    oWebClient:Headers:Set("Authorization", "Bearer " + accessToken).
    lcResult = oWebClient:DownloadString (vURL) . 
     
    Util.DemoUtils:BigMessage(lcResult).

    /* parse the json into a progress TT using the Progress Parser */ 
    oParse = NEW ObjectModelParser().
    oJSON = CAST (oParse:Parse(lcResult),JsonObject).

    /* EVERY table in SFDC has an "ID" field - if there is no ID in the JSON, something broke */
    IF oJSON:has("Id") THEN
    DO:
      /* load the JSON into the Temp-table */
      hBuff = hTT:DEFAULT-BUFFER-HANDLE.
      hBuff:BUFFER-CREATE ().
      /* loop throught the TT fields and copy in the data from the JSON */
      DO vi = 1 TO hBuff:NUM-FIELDS:
        hBuff:BUFFER-FIELD (vi):BUFFER-VALUE = oJSON:GetJsonText(hBuff:BUFFER-FIELD (vi):SERIALIZE-NAME) NO-ERROR.
      END.
    END.
    ELSE
      ErrorOut("Unknown error getting " + hTT:SERIALIZE-NAME). 
      
    RETURN.
  
  END METHOD.

  /*------------------------------------------------------------------------------
      Purpose:                                                                        
      Notes:                                                                      
  ------------------------------------------------------------------------------*/

  METHOD PUBLIC CHARACTER deleteSObjects(  ):
    DEFINE VARIABLE lcSObject AS LONGCHAR  NO-UNDO.
    DEFINE VARIABLE vcId      AS CHARACTER NO-UNDO.
    DEFINE VARIABLE lSuccess  AS LOGICAL   NO-UNDO.
    DEFINE VARIABLE vcMessage AS CHARACTER NO-UNDO.
   
    /* build the sObject XML from ttID */
    lcSObject = GenerateDeleteSObjectXML().
    
    Util.DemoUtils:BigXMLMessage(lcSObject). 

    RUN deleteSObjectFromTT IN hSFDCSOAPAPI
      (INPUT lcSObject,
      OUTPUT lSuccess,
      OUTPUT vcMessage).

    IF lSuccess THEN
      RETURN "".
    ELSE 
    DO:
      ErrorOut("Error deleteing SObject: " + (IF vcMessage = ? THEN "" ELSE vcMessage) ).
    END.

  END METHOD.
 
  /*------------------------------------------------------------------------------
     Purpose:                                                                        
     Notes:                                                                      
 ------------------------------------------------------------------------------*/
   
  METHOD PUBLIC VOID emptyTT (htt AS HANDLE):
    DEFINE VARIABLE hBH AS HANDLE NO-UNDO.
    hBH = htt:DEFAULT-BUFFER-HANDLE.
    hBH:EMPTY-TEMP-TABLE ().
  END METHOD.
  
  /*------------------------------------------------------------------------------
      Purpose:                                                                        
      Notes:                                                                      
  ------------------------------------------------------------------------------*/
  
  METHOD PUBLIC VOID ErrorOut( pMessage AS CHARACTER  ):
    UNDO, THROW NEW Progress.Lang.AppError(pMessage, 1969).
  END METHOD.

  /*------------------------------------------------------------------------------
      Purpose:                                                                        
      Notes:                                                                      
  ------------------------------------------------------------------------------*/

  METHOD PRIVATE LONGCHAR generateDeleteSObjectXML(  ):
    /* ID's must be unique across all of SFDC since this call doesn't require you to 
        provide the table name.  Pre-populate ttId with a list of SFDC Ids to delete.   */
    DEFINE VARIABLE lcXML      AS LONGCHAR NO-UNDO.
    DEFINE VARIABLE hSaxWriter AS HANDLE   NO-UNDO.
    DEFINE VARIABLE vi         AS INTEGER  NO-UNDO. 

    CREATE SAX-WRITER hSaxWriter.
    hSaxWriter:ENCODING = "UTF-8".
    hSaxWriter:FORMATTED = TRUE.
    hSaxWriter:SET-OUTPUT-DESTINATION("LONGCHAR",lcXML).
    hSaxWriter:START-DOCUMENT().    
    hSaxWriter:START-ELEMENT("sfdc:delete").
    addXmlsnsAttributes(hSaxWriter).
  
    FOR EACH ttId:
      vi = vi + 1.
      hSaxWriter:START-ELEMENT("sfdc:ids").
      hSaxWriter:WRITE-CHARACTERS(ttId.id).
      hSaxWriter:END-ELEMENT("sfdc:ids").
    END.
    
    hSaxWriter:END-ELEMENT("sfdc:delete").
    hSaxWriter:END-DOCUMENT().

    DELETE OBJECT hSaxWriter NO-ERROR.
    RETURN lcXML.
  END METHOD.

  /*------------------------------------------------------------------------------
      Purpose:                                                                        
      Notes:                                                                      
  ------------------------------------------------------------------------------*/

  METHOD PRIVATE LONGCHAR generateGetUpdatedXML( pTable AS CHARACTER, pStartDate AS CHARACTER, pEndDate AS CHARACTER ):
    DEFINE VARIABLE lcXML      AS LONGCHAR NO-UNDO.
    DEFINE VARIABLE hSaxWriter AS HANDLE   NO-UNDO.
    DEFINE VARIABLE vi         AS INTEGER  NO-UNDO. 

    CREATE SAX-WRITER hSaxWriter.
    hSaxWriter:ENCODING = "UTF-8".
    hSaxWriter:FORMATTED = TRUE.
    hSaxWriter:SET-OUTPUT-DESTINATION("LONGCHAR",lcXML).
    hSaxWriter:START-DOCUMENT().    
    hSaxWriter:START-ELEMENT("sfdc:getUpdated").
    addXmlsnsAttributes(hSaxWriter).
    
    hSaxWriter:START-ELEMENT("urn:sObjects").
    hSaxWriter:WRITE-CHARACTERS(pTable). 
    hSaxWriter:END-ELEMENT("urn:sObjects"). 
    hSaxWriter:START-ELEMENT("urn:startDate").
    hSaxWriter:WRITE-CHARACTERS(pStartDate). 
    hSaxWriter:END-ELEMENT("urn:startDate").    
    hSaxWriter:START-ELEMENT("urn:endDate").
    hSaxWriter:WRITE-CHARACTERS(pEndDate). 
    hSaxWriter:END-ELEMENT("urn:endDate").       

    hSaxWriter:END-ELEMENT("sfdc:getUpdated"). 
    hSaxWriter:END-DOCUMENT().

    DELETE OBJECT hSaxWriter NO-ERROR.
    RETURN lcXML.

  END METHOD.

  /*------------------------------------------------------------------------------
      Purpose:                                                                        
      Notes:                                                                      
  ------------------------------------------------------------------------------*/

  METHOD PRIVATE LONGCHAR generateMergeXML( hTT AS HANDLE, pTable AS CHARACTER ):
    DEFINE VARIABLE lcXML      AS LONGCHAR  NO-UNDO.
    DEFINE VARIABLE hSaxWriter AS HANDLE    NO-UNDO.
    DEFINE VARIABLE vcId       AS CHARACTER NO-UNDO.
    DEFINE VARIABLE hBuf       AS HANDLE    NO-UNDO.
    DEFINE VARIABLE hQry       AS HANDLE    NO-UNDO.
    
    CREATE SAX-WRITER hSaxWriter.
    hSaxWriter:ENCODING = "UTF-8".
    hSaxWriter:FORMATTED = TRUE.
    hSaxWriter:SET-OUTPUT-DESTINATION("LONGCHAR",lcXML).
    hSaxWriter:START-DOCUMENT().   

    hSaxWriter:START-ELEMENT ("urn:merge").
    addXmlsnsAttributes(hSaxWriter).
    hSaxWriter:START-ELEMENT ("urn:request").
    hSaxWriter:START-ELEMENT("urn:masterRecord").
    hSaxWriter:INSERT-ATTRIBUTE("xsi:type", "urn1:" + pTable).
    
    hBuf = hTT:DEFAULT-BUFFER-HANDLE.
    CREATE QUERY hQry.
    hQry:ADD-BUFFER(hBuf).
    hQry:QUERY-PREPARE('for each ' + hBuf:NAME).
    hQry:QUERY-OPEN().
    hQry:GET-FIRST().
    DO WHILE NOT hQry:QUERY-OFF-END:
      hSaxWriter:START-ELEMENT("urn1:Id").
      vcId = hBuf:BUFFER-FIELD ("id"):BUFFER-VALUE.
      hSaxWriter:WRITE-CHARACTERS(vcId). 
      hSaxWriter:END-ELEMENT("urn1:Id"). 
      hQry:GET-NEXT ().  
    END.
    hSaxWriter:END-ELEMENT("urn:masterRecord"). 
    
    hQry:GET-FIRST().
    DO WHILE NOT hQry:QUERY-OFF-END:
      hSaxWriter:START-ELEMENT("urn:recordToMergeIds").
      vcId = hBuf:BUFFER-FIELD ("fromId"):BUFFER-VALUE.
      hSaxWriter:WRITE-CHARACTERS(vcId). 
      hSaxWriter:END-ELEMENT("urn:recordToMergeIds"). 
      hQry:GET-NEXT ().
    END.

    hSaxWriter:END-ELEMENT("urn:request").
    hSaxWriter:END-ELEMENT("urn:merge").
    hSaxWriter:END-DOCUMENT().

    DELETE OBJECT hSaxWriter NO-ERROR.
    DELETE OBJECT hQry NO-ERROR.

    RETURN lcXML.

  END METHOD.

  /*------------------------------------------------------------------------------
      Purpose:                                                                        
      Notes:                                                                      
  ------------------------------------------------------------------------------*/

  METHOD PRIVATE LONGCHAR generateRetrieveToTTXML( hTT AS HANDLE):       
    DEFINE VARIABLE lcXML      AS LONGCHAR NO-UNDO.
    DEFINE VARIABLE hSaxWriter AS HANDLE   NO-UNDO.
    DEFINE VARIABLE vi         AS INTEGER  NO-UNDO. 
    
    CREATE SAX-WRITER hSaxWriter.
    hSaxWriter:ENCODING = "UTF-8".
    hSaxWriter:FORMATTED = TRUE.
    hSaxWriter:SET-OUTPUT-DESTINATION("LONGCHAR",lcXML).
    hSaxWriter:START-DOCUMENT().    
    hSaxWriter:START-ELEMENT("urn:retrieve").
    
    /* Attributes added to the start of every SFDC XML request */
    addXmlsnsAttributes(hSaxWriter).
  
    hSaxWriter:START-ELEMENT("urn:fieldList").
    hSaxWriter:WRITE-CHARACTERS(buildFieldList(htt)). 
    hSaxWriter:END-ELEMENT("urn:fieldList").
    
    hSaxWriter:START-ELEMENT("urn:sObjectType").
    hSaxWriter:WRITE-CHARACTERS(hTT:SERIALIZE-NAME).
    hSaxWriter:END-ELEMENT("urn:sObjectType").
  
    foreach:
    FOR EACH ttId:
      vi = vi + 1.
      IF vi > 199 THEN LEAVE foreach.
      hSaxWriter:START-ELEMENT("urn:ids").
      hSaxWriter:WRITE-CHARACTERS(ttId.id).
      hSaxWriter:END-ELEMENT("urn:ids").
    END.
    
    hSaxWriter:END-ELEMENT("urn:retrieve").
    hSaxWriter:END-DOCUMENT().

    DELETE OBJECT hSaxWriter NO-ERROR.
    RETURN lcXML.

  END METHOD.


  METHOD PRIVATE LONGCHAR generateUpdateCreateXML( pMode AS CHARACTER , hTT AS HANDLE ):
    /*------------------------------------------------------------------------------
                                                                                
      ------------------------------------------------------------------------------*/
    DEFINE VARIABLE lcXML        AS LONGCHAR  NO-UNDO.
    DEFINE VARIABLE hSaxWriter   AS HANDLE    NO-UNDO.
    DEFINE VARIABLE vi           AS INTEGER   NO-UNDO.
    DEFINE VARIABLE viFields     AS INTEGER   NO-UNDO.
    DEFINE VARIABLE vcId         AS CHARACTER NO-UNDO.
    DEFINE VARIABLE hBuf         AS HANDLE    NO-UNDO.
    DEFINE VARIABLE hField       AS HANDLE    NO-UNDO.
    DEFINE VARIABLE vcFieldValue AS CHARACTER NO-UNDO.
    DEFINE VARIABLE vExists      AS LOGICAL   NO-UNDO.
    DEFINE VARIABLE hQry         AS HANDLE    NO-UNDO.
    DEFINE VARIABLE hQry2        AS HANDLE    NO-UNDO.
    
    CREATE SAX-WRITER hSaxWriter.
    hSaxWriter:ENCODING = "UTF-8".
    hSaxWriter:FORMATTED = TRUE.
    hSaxWriter:SET-OUTPUT-DESTINATION("LONGCHAR",lcXML).
    hSaxWriter:START-DOCUMENT().   
    hSaxWriter:START-ELEMENT("sfdc:" + pMode).
    
    /* add the XML NameSpace attributes */
    addXmlsnsAttributes(hSaxWriter).
  
    hBuf = hTT:DEFAULT-BUFFER-HANDLE.
    CREATE QUERY hQry.
    hQry:ADD-BUFFER(hBuf).
    hQry:QUERY-PREPARE('for each ' + hBuf:NAME).
    hQry:QUERY-OPEN().
    hQry:GET-FIRST().

    DO WHILE NOT hQry:QUERY-OFF-END:
      hSaxWriter:START-ELEMENT("urn:sObjects").
      hSaxWriter:INSERT-ATTRIBUTE("xsi:type","urn1:" + hTT:SERIALIZE-NAME).

      /* update only "stuff" */
      IF pMode = "update" THEN
      DO:          
        /* an update requires the ID of the record to be updated as first node */
        hSaxWriter:START-ELEMENT("urn1:Id").
        vcId = hBuf:BUFFER-FIELD ("id"):BUFFER-VALUE.
        hSaxWriter:WRITE-CHARACTERS(vcId). 
        hSaxWriter:END-ELEMENT("urn1:Id"). 
        
        /* Passing a node with a field name and no value does NOT blank out the value. You
           have to create a "fieldsToNull" node and put the field names as child nodes     */
        CREATE QUERY hQry2.
        hQry2:ADD-BUFFER (hTTFieldsToNull:DEFAULT-BUFFER-HANDLE).
        hQry2:QUERY-PREPARE ("for each ttFieldsToNull where ttFieldsToNull.id = '" + vcId + "' ").
        hQry2:QUERY-OPEN.
        hQry2:GET-FIRST ().
        /* we created a ttFieldsToNull for every field that needs to be blanked */
        DO WHILE NOT hQry2:QUERY-OFF-END:
          hSaxWriter:START-ELEMENT("urn1:fieldsToNull").
          vcFieldValue = hTTFieldsToNull:DEFAULT-BUFFER-HANDLE:BUFFER-FIELD ("fieldName"):BUFFER-VALUE. 
          hSaxWriter:WRITE-CHARACTERS(vcFieldValue). 
          hSaxWriter:END-ELEMENT("urn1:fieldsToNull"). 
          hQry2:GET-NEXT ().
        END.
      END.  /* pmode = "update" */
         
      /* loop through all fields in the TT and create an xml node for ones with values */      
      viFields = hBuf:NUM-FIELDS.       
          
      fieldLoop:
      DO vi = 1 TO viFields:
        hfield = hBuf:BUFFER-FIELD (vi).
        IF hField:SERIALIZE-NAME = "Id"  /* We can never insert or update the primary SFDC key field */
          OR  hField:SERIALIZE-NAME = "skipSFDC"
          OR hField:BUFFER-VALUE = ? THEN
          NEXT fieldLoop.
                
        CASE hField:DATA-TYPE:
          WHEN "logical" THEN
            DO:
              vcFieldValue = STRING (hField:BUFFER-VALUE , "true/false").
            END.
          WHEN "character" THEN
            DO:
              IF hField:BUFFER-VALUE = "" THEN 
                NEXT fieldLoop.
              vcFieldValue = hField:BUFFER-VALUE.
            END.
          OTHERWISE 
          vcfieldValue = hfield:BUFFER-VALUE.
        END CASE.
        
        /* write out each field as a node using the serialize-name as the name */
        hSaxWriter:START-ELEMENT(hfield:SERIALIZE-NAME).
        hSaxWriter:WRITE-CHARACTERS(vcFieldValue). 
        hSaxWriter:END-ELEMENT(hfield:SERIALIZE-NAME).            
      END.
     
      hSaxWriter:END-ELEMENT("urn:sObjects").
      hQry:GET-NEXT(). 
    END. 
    hSaxWriter:END-ELEMENT("sfdc:" + pMode). 
    hSaxWriter:END-DOCUMENT().

    DELETE OBJECT hSaxWriter NO-ERROR.
    DELETE OBJECT hQry NO-ERROR.

    RETURN lcXML.

  END METHOD.

  /*------------------------------------------------------------------------------
      Purpose:                                                                        
      Notes:                                                                      
  ------------------------------------------------------------------------------*/

  METHOD PUBLIC CHARACTER  getUpdated( pTable AS CHARACTER, startDT AS DATETIME-TZ, endDT AS DATETIME-TZ ):
        
    DEFINE VARIABLE vcStart    AS CHARACTER NO-UNDO.
    DEFINE VARIABLE vcEnd      AS CHARACTER NO-UNDO. 
    DEFINE VARIABLE lcResult   AS LONGCHAR  NO-UNDO.
    DEFINE VARIABLE vcLastDate AS CHARACTER NO-UNDO.     
    
    vcStart = ISO-DATE (startDT).
    vcEnd = ISO-DATE (endDT).
    lcResult = GenerateGetUpdatedXML (pTable,vcStart, vcEnd).

    emptyTT(hTTId).
    
    RUN getUpdated IN hSFDCSOAPAPI
      (INPUT lcResult,
      INPUT SessionId,
      INPUT hTTId,
      OUTPUT vcLastDate). 

    RETURN vcLastDate.

  END METHOD.

  /*------------------------------------------------------------------------------
      Purpose:                                                                        
      Notes:                                                                      
  ------------------------------------------------------------------------------*/

  METHOD PUBLIC CHARACTER getValue(pFieldName AS CHARACTER, pOSQL AS CHARACTER ):
    DEFINE VARIABLE vcValue AS CHARACTER NO-UNDO.  
    RUN getValue IN hSFDCSOAPAPI (pFieldName, pOSQL, OUTPUT vcValue). 
    RETURN vcValue.
  END METHOD.
  
  /*------------------------------------------------------------------------------
     Purpose:                                                                        
     Notes:                                                                      
 ------------------------------------------------------------------------------*/
  METHOD PUBLIC CHARACTER insertSObjectFromTT( hTT AS HANDLE  ):
    DEFINE VARIABLE lcSObject AS LONGCHAR  NO-UNDO.
    DEFINE VARIABLE vcId      AS CHARACTER NO-UNDO.
    DEFINE VARIABLE lSuccess  AS LOGICAL   NO-UNDO.
    DEFINE VARIABLE vcError   AS CHARACTER NO-UNDO.
 
    /* generate the SOAP XML dynamically from the TT so that you don't have to know the XML */
    lcSObject = GenerateUpdateCreateXML("create", hTT).
      
    /* DEMO!! -----   show the XML */
    Util.DemoUtils:BigXMLMessage(lcSObject). 

    RUN insertSObjectFromTT IN hSFDCSOAPAPI
      (INPUT lcSObject,
      OUTPUT vcId,
      OUTPUT lSuccess,
      OUTPUT vcError).

    IF lSuccess THEN
      RETURN vcId.
    ELSE 
    DO:
      IF vcError <> "" THEN 
        ErrorOut (vcError).  
      RETURN ?.
    END.

  END METHOD.

  /*------------------------------------------------------------------------------
      Purpose:                                                                        
      Notes:                                                                      
  ------------------------------------------------------------------------------*/
 
  METHOD PUBLIC VOID loginToSFDC():
    loginToSFDC(TRUE).
  END METHOD. 
 
  METHOD PUBLIC VOID loginToSFDC(pUseSoap AS LOGICAL ):
    DEFINE VARIABLE vError AS CHARACTER NO-UNDO.
    DEFINE VARIABLE vWSDL  AS CHARACTER NO-UNDO.

    vWSDL = SEARCH ("SFDCEnterprise.wsdl").

    RUN loginToSFDC IN hSFDCSOAPAPI
      (INPUT vWSDL, 
      OUTPUT SessionId, 
      OUTPUT SoapEndPoint, 
      OUTPUT vError). 
    
    IF vError <> "" THEN 
      ErrorOut (vError).
        
    /* Build the instance URL used by the Bulk API */  
    instanceURL = "https://" + ENTRY (3, SoapEndPoint, "/").
    
    IF pUseSoap THEN 
      /* login to your instance of SFDC using the returned endpoint and set
         the callback procedure for setting the header with the sessionId */
      RUN SOAPConnect IN hSFDCSOAPAPI
        (INPUT vWSDL, INPUT SoapEndPoint, INPUT SessionId ).
        
    RETURN.
  END METHOD.
  
  /*------------------------------------------------------------------------------
      Purpose:                                                                        
      Notes:                                                                      
  ------------------------------------------------------------------------------*/

  METHOD PUBLIC VOID loginToSFDCREST(  ):
    DEFINE VARIABLE consumerKey    AS CHARACTER           NO-UNDO.
    DEFINE VARIABLE consumerSecret AS CHARACTER           NO-UNDO.
    DEFINE VARIABLE securityToken  AS CHARACTER           NO-UNDO.
    DEFINE VARIABLE vURL           AS CHARACTER           NO-UNDO.
    DEFINE VARIABLE vPassword      AS CHARACTER           NO-UNDO.
    DEFINE VARIABLE jsonResponse   AS LONGCHAR            NO-UNDO.
    DEFINE VARIABLE oParse         AS ObjectModelParser.
    DEFINE VARIABLE oJSON          AS JsonObject.
    DEFINE VARIABLE oWebClient     AS WebClient           NO-UNDO.  
    DEFINE VARIABLE oCollection    AS NameValueCollection NO-UNDO.  
    DEFINE VARIABLE oByte          AS "System.Byte[]"     NO-UNDO.

    consumerKey = "3MVG9rFJvQRVOvk6wuwY2KY__YzlgQ5zxyDwtJfAZzkOrwD2PdcKwuHZhlOAdZ0rgCTUfE5yxDWHihKczB3sY".
    consumerSecret = "8740385280641706855".
    securityToken = "iDh73C04OeACMHZA6g5hyfNQ".
    vURL = "https://login.salesforce.com/services/oauth2/token".
    GET-KEY-VALUE SECTION "PUGChallenge" KEY "Password" VALUE vPassword.
    
    oWebClient = NEW WebClient().
    oWebClient:Encoding = Encoding:UTF8.

    /* we want to use a grant_type of "password"  rather than "authorization_code"
       so that we can get the token returned instead of using OAuth 2.0 */
    oWebClient:QueryString:Add("grant_type", "password"). 
    oWebClient:QueryString:Add ("client_id", consumerKey).
    oWebClient:QueryString:Add ("client_secret", consumerSecret).
    oWebClient:QueryString:Add ("username", "gwinning@gmail.com.dev").
    oWebClient:QueryString:Add ("password", vPassword + securityToken).

    jsonResponse = oWebClient:UploadString(vURL, "Post", "") .
    Util.DemoUtils:BigMessage (jsonResponse). 
    /* is there a better way to get the JSON text into a progress json object?? */
    oParse = NEW Progress.Json.ObjectModel.ObjectModelParser() .
    oJSON = CAST (oParse:Parse(jsonResponse),JsonObject) .
  
    instanceURL = REPLACE (oJSON:GetJSONText("instance_url"), "\", "").                     
    accessToken = oJSON:GetJSONText("access_token"). 
 
    Util.DemoUtils:BigMessage("InstanceURL: " + instanceURL + "~n" + "Access Token: " + accessToken). 
    RETURN.
  END METHOD.

  /*------------------------------------------------------------------------------
      Purpose:                                                                        
      Notes:                                                                      
  ------------------------------------------------------------------------------*/
  
  METHOD PUBLIC VOID logOutFromSFDC():
    /* WARNING - EVERY session for this SFDC user will be logged out, not just this sessionId */
    RUN logOutFromSFDC IN hSFDCSOAPAPI (SessionId).
    RETURN.    
  END METHOD.

  /*------------------------------------------------------------------------------
      Purpose:    format date -    1969-09-20                                                                 
      Notes:                                                                      
  ------------------------------------------------------------------------------*/

  METHOD PUBLIC CHARACTER makeDateString( pDate AS DATE ):
    DEFINE VARIABLE vc AS CHARACTER NO-UNDO.
        
    IF pDate = ? THEN
      RETURN "". 
      
    vc = STRING (YEAR (pdate), "9999") + "-" + STRING (MONTH (pDate),"99") + "-" + STRING (DAY (pDate), "99").
    RETURN vc. 

  END METHOD.

  /*------------------------------------------------------------------------------
      Purpose:                                                                        
      Notes:                                                                      
  ------------------------------------------------------------------------------*/

  METHOD PUBLIC CHARACTER mergeSObjects( pTable AS CHARACTER  ):
    DEFINE VARIABLE vcId      AS CHARACTER NO-UNDO.
    DEFINE VARIABLE result    AS CHARACTER NO-UNDO.
    DEFINE VARIABLE lcResult  AS LONGCHAR  NO-UNDO.
    DEFINE VARIABLE lSuccess  AS LOGICAL   NO-UNDO.
    DEFINE VARIABLE vcMEssage AS CHARACTER NO-UNDO.     
    
    lcResult = GenerateMergeXML(hTTId, pTable).

    RUN mergeSObject IN hSFDCSOAPAPI
      (INPUT lcResult,
      INPUT SessionId,
      OUTPUT vcId,
      OUTPUT lSuccess,
      OUTPUT vcMessage).

    IF lSuccess THEN
      RETURN vcId.
    ELSE 
    DO:
      PUT UNFORMATTED NOW " Error merging: "  vcMessage SKIP .
      RETURN ?.
    END.

    RETURN result.

  END METHOD.

  /*------------------------------------------------------------------------------
      Purpose:                                                                        
      Notes:                                                                      
  ------------------------------------------------------------------------------*/

  METHOD PUBLIC VOID queryToTT(hTT AS HANDLE, pWhere AS CHARACTER):
    DEFINE VARIABLE vOSQL AS CHARACTER NO-UNDO.

    vOSQL = "Select " + buildFieldList(hTT) + " from " + hTT:SERIALIZE-NAME + " " + pWhere.
    Util.DemoUtils:BigMessage(vOSQL).
    
    RUN queryToTT IN hSFDCSOAPAPI (vOSQL, hTT).
    RETURN.

  END METHOD.

  /*------------------------------------------------------------------------------
      Purpose:                                                                        
      Notes:                                                                      
  ------------------------------------------------------------------------------*/

  METHOD PUBLIC VOID retrieveToTT( hTable AS HANDLE ):
    DEFINE VARIABLE lcResult  AS LONGCHAR  NO-UNDO.
    DEFINE VARIABLE vcId      AS CHARACTER NO-UNDO.
    DEFINE VARIABLE lSuccess  AS LOGICAL   NO-UNDO.
    DEFINE VARIABLE vcMessage AS CHARACTER NO-UNDO.
    
    /*   this will populate htable with
       the records in SFDC with those IDs (serialize-name of hTable is the SFDC table name  */
    lcResult = generateRetrieveToTTXML(hTable).
    
    Util.DemoUtils:BigXMLMessage(lcResult).

    RUN retrieveToTT IN hSFDCSOAPAPI
      (INPUT lcResult,
      INPUT SessionId,
      INPUT hTable).

    RETURN.

  END METHOD.

  /*------------------------------------------------------------------------------
      Purpose:                                                                        
      Notes:                                                                      
  ------------------------------------------------------------------------------*/

  METHOD PUBLIC VOID updateSObjectFromTT(hTT AS HANDLE):
    DEFINE VARIABLE lcSObject AS LONGCHAR  NO-UNDO.
    DEFINE VARIABLE vcId      AS CHARACTER NO-UNDO.
    DEFINE VARIABLE lSuccess  AS LOGICAL   NO-UNDO.
    DEFINE VARIABLE vcMessage AS CHARACTER NO-UNDO.
    DEFINE VARIABLE vExists   AS LOGICAL   NO-UNDO.
                 
    /* Generate the XML that represents the update */        
    lcSObject = GenerateUpdateCreateXML("update", hTT). 
    
    Util.DemoUtils:BigXMLMessage(lcSObject). 
     
    RUN updateSObjectFromTT IN hSFDCSOAPAPI
      (INPUT lcSObject,
      OUTPUT lSuccess,
      OUTPUT vcMessage).
      
    IF NOT lSuccess THEN
    DO:
      ErrorOut (vcMessage). 
      RETURN.
    END.

    RETURN.

  END METHOD.

  /*------------------------------------------------------------------------------
   Purpose:
   Notes:
  ------------------------------------------------------------------------------*/
  DESTRUCTOR PUBLIC SFDCLib ( ):
    DELETE PROCEDURE hSFDCSOAPAPI NO-ERROR.
  END DESTRUCTOR.

END CLASS.
