/*------------------------------------------------------------------------
   File        : SFDCAPI.p
   Author(s)   : Gerry Winning
                 gwinning@gmail.com
 ---------------------------------------------------------------------- */

BLOCK-LEVEL ON ERROR UNDO, THROW.   

USING Util.*.
USING System.Net.*.
USING System.Text.*.

DEFINE VARIABLE lcResult        AS LONGCHAR  NO-UNDO.
DEFINE VARIABLE vcSFDCSession   AS CHARACTER NO-UNDO.
DEFINE VARIABLE vcSFDCserverUrl AS CHARACTER NO-UNDO.
DEFINE VARIABLE fldData         AS CHARACTER NO-UNDO.
DEFINE VARIABLE hParser         AS HANDLE    NO-UNDO.
DEFINE VARIABLE hSOAPHeader     AS HANDLE    NO-UNDO.
DEFINE VARIABLE vcMode          AS CHARACTER NO-UNDO.
DEFINE VARIABLE vcId            AS CHARACTER NO-UNDO.
DEFINE VARIABLE vcValue         AS CHARACTER NO-UNDO.
DEFINE VARIABLE vFieldName      AS CHARACTER NO-UNDO.
DEFINE VARIABLE vcSuccess       AS CHARACTER NO-UNDO.
DEFINE VARIABLE vcMessage       AS CHARACTER NO-UNDO.
DEFINE VARIABLE hTT             AS HANDLE    NO-UNDO.
DEFINE VARIABLE hWebService     AS HANDLE    NO-UNDO.
DEFINE VARIABLE hSoap           AS HANDLE    NO-UNDO.


FUNCTION getLocalFieldName RETURNS CHARACTER 
  ( INPUT SFDCField AS CHARACTER,
  INPUT hTT AS HANDLE  ) FORWARD.

/* **********************  Internal Procedures  *********************** */


PROCEDURE createSoapHeader:
  /*------------------------------------------------------------------------------
      Purpose:                                      
      Notes:                                      
  ------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER pSessionId AS CHARACTER NO-UNDO. 
  
  DEFINE VARIABLE hSOAPEntry AS HANDLE NO-UNDO.
  DEFINE VARIABLE hDoc       AS HANDLE NO-UNDO.
  DEFINE VARIABLE hRoot      AS HANDLE NO-UNDO.
  DEFINE VARIABLE hElement   AS HANDLE NO-UNDO.
  DEFINE VARIABLE hValue     AS HANDLE NO-UNDO.
  
  CREATE SOAP-HEADER hSoapHeader. 
  CREATE SOAP-HEADER-ENTRYREF hSOAPEntry.
  hSoapHeader:ADD-HEADER-ENTRY( hSOAPEntry ).

  /* This is the only place I use the DOM parser
     because I stole the code from the Progress examples  */
  CREATE X-DOCUMENT hDoc.
  CREATE X-NODEREF hRoot.
  CREATE X-NODEREF hElement.
  CREATE X-NODEREF hValue.

  hDoc:CREATE-NODE-NAMESPACE( hRoot, 'urn:enterprise.soap.sforce.com', "SforceService:SessionHeader", "element" ).
  hDoc:APPEND-CHILD( hRoot ).
  hRoot:SET-ATTRIBUTE( "xmlns:SforceService", 'urn:enterprise.soap.sforce.com' ).
    
  hDoc:CREATE-NODE-NAMESPACE( hElement, 'urn:enterprise.soap.sforce.com', "SforceService:sessionId", "element" ).
  hDoc:CREATE-NODE( hValue, ?, "text" ).
  hRoot:APPEND-CHILD( hElement ).
  hElement:APPEND-CHILD( hValue ).
  hValue:NODE-VALUE = pSessionID.
  hSOAPEntry:SET-NODE( hRoot ).

  DELETE OBJECT hSOAPEntry NO-ERROR.
  DELETE OBJECT hDoc NO-ERROR.
  DELETE OBJECT hRoot NO-ERROR.
  DELETE OBJECT hElement NO-ERROR.
  DELETE OBJECT hValue NO-ERROR.
   
END PROCEDURE.


PROCEDURE characters:
  /*------------------------------------------------------------------------------
    Purpose:     
    Parameters:  <none>
    Notes:       
  ------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER charData AS MEMPTR.
  DEFINE INPUT PARAMETER numChars AS INTEGER.

  fldData = fldData + GET-STRING(charData,1).
END PROCEDURE.

PROCEDURE deleteSObjectFromTT:
  /*------------------------------------------------------------------------------
      Purpose:                                      
      Notes:                                      
  ------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER lcSObject AS LONGCHAR NO-UNDO.
  DEFINE OUTPUT PARAMETER pSuccess AS LOGICAL NO-UNDO.
  DEFINE OUTPUT PARAMETER pMessage AS CHARACTER NO-UNDO.
    
  /* pass the sObjects XML to the SOAP web service */
  RUN delete IN hSoap(INPUT lcSObject, OUTPUT lcResult).
  
  Util.DemoUtils:BigXMLMessage(lcResult). 
   
  vcMode = "DeleteSObject".
  RUN parseResult.
  IF vcSuccess = 'true' THEN 
    pSuccess = TRUE. 
  ELSE 
    ASSIGN
      pSuccess = FALSE
      pMessage = vcMessage.
          
  RETURN.
END PROCEDURE.

PROCEDURE getUpdated:
  /*------------------------------------------------------------------------------
      Purpose:                                                                        
      Notes:                                                                      
  ------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER lcRequest AS LONGCHAR NO-UNDO.
  DEFINE INPUT PARAMETER pSessionId AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER phTT AS HANDLE NO-UNDO.
  DEFINE OUTPUT PARAMETER pLastDate AS CHARACTER NO-UNDO.
      
  htt = phTT.  
  RUN getUpdated IN hSoap(INPUT lcRequest, OUTPUT lcResult).  
  vcMode = "getUpdated".
  RUN parseResult.
  pLastDate = vcMessage.

  RETURN.
END PROCEDURE.


PROCEDURE getValue:
  /*------------------------------------------------------------------------------
      Purpose:                                                                        
      Notes:                                                                      
  ------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER pFieldName AS CHARACTER NO-UNDO. 
  DEFINE INPUT PARAMETER pOSQL AS CHARACTER   NO-UNDO.
  DEFINE OUTPUT PARAMETER pValue AS CHARACTER NO-UNDO.

  /* In the WSDL -  <operation name="query"> */
  RUN query IN hSoap(INPUT pOSQL, OUTPUT lcResult).
  
  Util.DemoUtils:BigXMLMessage(lcResult).
  
  vcValue = ?.
  vFieldName = pFieldName. 
  vcMode = "getValue".
  RUN parseResult.
  pValue = vcValue.
  
  RETURN.
END PROCEDURE.

PROCEDURE insertSObjectFromTT:
  /*------------------------------------------------------------------------------
      Purpose:                                      
      Notes:                                      
  ------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER lcSObject AS LONGCHAR NO-UNDO.
  DEFINE OUTPUT PARAMETER pId AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER pSuccess AS LOGICAL NO-UNDO.
  DEFINE OUTPUT PARAMETER pMessage AS CHARACTER NO-UNDO.
      
  /* pass the sObjects XML to the SOAP web service */
  RUN create IN hSoap(INPUT lcSObject, OUTPUT lcResult).
  
  DemoUtils:BigXMLMessage(lcResult).

  /* parse the results */
  vcMode = "CreateSObject".
  vcId = ?.
  RUN parseResult.
   
  IF vcSuccess = 'true' THEN 
    ASSIGN 
      pSuccess = TRUE
      pId      = vcId
      pMessage = "".
  ELSE 
    ASSIGN
      pSuccess = FALSE
      pId      = ?
      pMessage = vcMessage.
  RETURN.
  
END PROCEDURE.

PROCEDURE QueryToTT:
  /*------------------------------------------------------------------------------
      Purpose:                                      
      Notes:                                      
  ------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER psoql      AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER phtt        AS HANDLE NO-UNDO.
  
  hTT = phtt.
  RUN query IN hSoap(INPUT psoql, OUTPUT lcResult) .

  Util.DemoUtils:BigXMLMessage(lcResult).

  vcMode = "queryToTT".
  RUN parseResult.
  RETURN.
  
END PROCEDURE.

PROCEDURE endElement:
  /* callback procedure - SAX Parser */
  DEFINE INPUT PARAMETER namespaceURI AS CHARACTER.
  DEFINE INPUT PARAMETER localName    AS CHARACTER.
  DEFINE INPUT PARAMETER qname        AS CHARACTER.
    
  fldData = TRIM(fldData).
  
  CASE vcMode:
    WHEN "login" THEN 
      DO:
        CASE localName:
          WHEN "SessionId" THEN 
            vcSFDCSession = fldData.
          WHEN "serverUrl" THEN 
            vcSFDCserverUrl = fldData.
          WHEN "userid" THEN
            vcId = fldData.
        END.
      END.
    WHEN "getValue" THEN
      DO:
        IF localName = vFieldName THEN 
          vcValue = fldData.
      END.
    WHEN "RESTSoapLogin" THEN
      DO:
        CASE localName:
          WHEN "sessionId" THEN
            vcSFDCSession = fldData.
          WHEN "serverURL" THEN
            vcSFDCserverUrl = fldData.
          WHEN "faultstring" THEN
            vcMessage = fldData.
        END CASE.
      END.
    WHEN "CreateSObject" OR 
    WHEN "UpdateSObject" OR 
    WHEN "DeleteSObject" THEN
      DO:
        CASE localName:
          WHEN "Id" THEN   
            vcId = fldData.
          WHEN "success" THEN 
            vcSuccess = fldData.
          WHEN "message" THEN
            vcMessage = fldData.   
        END CASE. 
      END. 
    WHEN "queryToTT"  OR
    WHEN "RetrieveToTT "THEN
      DO:
        hTT:DEFAULT-BUFFER-HANDLE:buffer-field (getLocalFieldName(localName, hTT)):BUFFER-VALUE = fldData NO-ERROR.  
      END.
    WHEN "getUpdated" THEN
      DO:
        CASE localName:
          WHEN "ids" THEN
            DO:
              hTT:DEFAULT-BUFFER-HANDLE:BUFFER-CREATE ().
              hTT:DEFAULT-BUFFER-HANDLE:buffer-field("id"):BUFFER-VALUE = fldData.
            END.
          WHEN "latestDateCovered" THEN
            ASSIGN 
              vcMessage = fldData.
        END CASE.
      END.        
  END CASE.
  fldData = "".
   
  RETURN.
   
END PROCEDURE.


PROCEDURE requestHandler:
  DEFINE OUTPUT   PARAMETER phSOAPHeader  AS HANDLE NO-UNDO.
  DEFINE INPUT    PARAMETER pcNamespace   AS CHARACTER NO-UNDO.
  DEFINE INPUT    PARAMETER pcLocalName   AS CHARACTER NO-UNDO.
  DEFINE OUTPUT   PARAMETER plDelete      AS LOG NO-UNDO.
 
  phSOAPHeader = hSOAPHeader.

END PROCEDURE. /* requestHandler */

PROCEDURE parseResult:
  /*------------------------------------------------------------------------------
      The SAX parser will call back to internal procedures 
      startElement, characters and endElement                                                                     
  ------------------------------------------------------------------------------*/

  CREATE SAX-READER hParser.
  hParser:HANDLER = THIS-PROCEDURE.
  hParser:SET-INPUT-SOURCE("longchar", lcResult).
  hParser:SAX-PARSE().
  DELETE OBJECT hParser NO-ERROR.
END PROCEDURE.


PROCEDURE retrieveToTT:
  /*------------------------------------------------------------------------------
      Purpose:                                                                        
      Notes:                                                                      
  ------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER lcRequest AS LONGCHAR NO-UNDO.
  DEFINE INPUT PARAMETER pSessionId AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER phTT AS HANDLE NO-UNDO.
      
  htt = phTT.  

  /* You must have already populated ttIds with the record IDs you wish to retreive */
  RUN retrieve IN hSoap(INPUT lcRequest, OUTPUT lcResult).  

  Util.DemoUtils:BigXMLMessage(lcResult).

  vcMode = "RetrieveToTT".
  RUN parseResult.
  RETURN.

END PROCEDURE.

PROCEDURE SOAPConnect:
  /*------------------------------------------------------------------------------
   Purpose:
   Notes:
  ------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER pWsdl AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER pSoapEndPoint AS CHARACTER NO-UNDO.
  DEFINE INPUT PARAMETER pSessionId AS CHARACTER NO-UNDO.

  /* reconnect using the endpoint returned from login */
  CREATE SERVER hWebService.
  hWebService:CONNECT(
    "-WSDL '" + pWSDL + "'" 
    + "-Binding SoapBinding "
    + "-nohostverify "   
    + "-SOAPEndpoint " +  pSoapEndPoint ) .
   
  RUN Soap SET hSoap ON hWebService .
    
  /* From now on, every call must have the sessionID passed in the header */
  /* Progress lets you create a soap-header object to pass to hSOAP */
  RUN createSoapHeader (INPUT pSessionId).     
  /* Every call using HSOAP will call this procedure first */
  hSoap:SET-CALLBACK-PROCEDURE ( "request-header", "requestHandler", THIS-PROCEDURE ).
  
END PROCEDURE.

PROCEDURE loginToSFDC:
  /*------------------------------------------------------------------------------
      Purpose:                                                                        
      Notes:                                                                      
  ------------------------------------------------------------------------------*/  
  DEFINE INPUT  PARAMETER pWSDL         AS CHARACTER NO-UNDO. 
  DEFINE OUTPUT PARAMETER pSFDCSession  AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER pSoapEndPoint AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER pError        AS CHARACTER NO-UNDO. 
  
  DEFINE VARIABLE username        AS CHARACTER NO-UNDO.
  DEFINE VARIABLE SecuredPassword AS CHARACTER NO-UNDO.  
  DEFINE VARIABLE securityToken   AS CHARACTER NO-UNDO.
  DEFINE VARIABLE vPassword       AS CHARACTER NO-UNDO. 
   
  username = "gwinning@gmail.com.Dev".
  /* SecurityToken must be fore this User - generated by SFDC */
  securityToken = "iDh73C04OeACMHZA6g5hyfNQ".
  
  GET-KEY-VALUE SECTION "PUGChallenge" KEY "Password" VALUE vPassword.
  /* SecrityToken always gets appended to the password */
  SecuredPassword = vPassword + securityToken.

  /* PROGRESS SOAP -
     Create the Server object,
     connect using the information in the WSDL (login.salesforce.com),
     set the handle to use the Progress SOAP object  */
  CREATE SERVER hWebService.
  hWebService:CONNECT("-WSDL '" + pWSDL + "'").  
  RUN Soap SET hSoap ON hWebService.
  
  /* Call the Login Method - this gets a valid sesion in SFDC (and retuns the id to this session + the redirect URL) */
  RUN login IN hSoap(
    INPUT username, 
    INPUT SecuredPassword, 
    OUTPUT lcResult) /* the XML response is returnedhere */
    NO-ERROR .
  IF ERROR-STATUS:NUM-MESSAGES > 0 THEN 
  DO: 
    pError = ERROR-STATUS:GET-MESSAGE (1).
    RETURN.
  END.
    
  /* Demo - show the resulting XML */
  COPY-LOB lcResult TO FILE "login.xml". 
    
  /* parse the xml returned to set vcSFDCSession  and vcSFDCserverUrl */  
  vcMode = 'login'.
  RUN parseResult.  
  
  /* Return the session and the serverURL to the class 
     We'll need them for all our calls to our instance of SFDC */
  pSFDCSession = vcSFDCSession.  
  pSoapEndPoint = vcSFDCserverUrl.
    
  /* disconnect - we are done with the login URL */
  hWebService:DISCONNECT( ).
  DELETE OBJECT hWebService NO-ERROR. 
    
  RETURN.
END.

PROCEDURE LogOutFromSFDC:
  /*------------------------------------------------------------------------------
      Purpose:                                      
      Notes:                                      
  ------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER pSessionId AS CHARACTER NO-UNDO.
    
  RUN logout IN hSoap.
  hWebService:DISCONNECT ().
  DELETE OBJECT hWebService NO-ERROR.
  DELETE OBJECT hSOAPHeader NO-ERROR. 
  RETURN.
    
END PROCEDURE.


PROCEDURE mergeSObject:
  /*------------------------------------------------------------------------------
      Purpose:                                      
      Notes:                                      
  ------------------------------------------------------------------------------*/

  DEFINE INPUT PARAMETER lcSObject AS LONGCHAR NO-UNDO.
  DEFINE OUTPUT PARAMETER pId AS CHARACTER NO-UNDO.
  DEFINE OUTPUT PARAMETER pSuccess AS LOGICAL NO-UNDO.
  DEFINE OUTPUT PARAMETER pMessage AS CHARACTER NO-UNDO.

  RUN merge IN hSoap(INPUT lcSObject, OUTPUT lcResult).

  /* parse the results */
  vcMode = "CreateSObject".
  RUN parseSaxResult.
   
  IF vcSuccess = 'true' THEN 
    ASSIGN 
      pSuccess = TRUE
      pId      = vcId
      pMessage = "".
  ELSE 
    ASSIGN
      pSuccess = FALSE
      pId      = ""
      pMessage = vcMessage.
  RETURN.
END PROCEDURE.
   
PROCEDURE startElement :
  /*------------------------------------------------------------------------------
    Purpose:     
    Parameters:  <none>
    Notes:       
  ------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER namespaceURI AS CHARACTER.
  DEFINE INPUT PARAMETER localName    AS CHARACTER.
  DEFINE INPUT PARAMETER qName        AS CHARACTER.
  DEFINE INPUT PARAMETER attributes   AS HANDLE.
 
  CASE vcMode:
    WHEN "queryToTT" THEN
      DO: 
        IF localName = "records" OR localName = "RetrieveToTT" THEN
        DO:
          hTT:DEFAULT-BUFFER-HANDLE:BUFFER-CREATE ().
        END.
      END.
  END CASE.

END PROCEDURE.

PROCEDURE updateSObjectFromTT:
  /*------------------------------------------------------------------------------
      Purpose:                                      
      Notes:                                      
  ------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER lcSObject AS LONGCHAR NO-UNDO.
  DEFINE OUTPUT PARAMETER pSuccess AS LOGICAL NO-UNDO.
  DEFINE OUTPUT PARAMETER pMessage AS CHARACTER NO-UNDO.
      
  RUN update IN hSoap(INPUT lcSObject, OUTPUT lcResult)  .

  Util.DemoUtils:BigXMLMessage(lcResult). 

  /* parse the results with the SAX parser*/
  vcMode = "UpdateSObject". 
  RUN parseResult.
   
  IF vcSuccess = 'true' THEN 
    ASSIGN 
      pSuccess = TRUE
      pMessage = "".
  ELSE 
    ASSIGN
      pSuccess = FALSE
      pMessage = vcMessage.
      
  RETURN.
END PROCEDURE.


/* ************************  Function Implementations ***************** */

FUNCTION getLocalFieldName RETURNS CHARACTER 
  ( INPUT SFDCField AS CHARACTER,
  INPUT hTT AS HANDLE  ):
  /*------------------------------------------------------------------------------
      Purpose:                                      
      Notes:                                      
  ------------------------------------------------------------------------------*/  
  DEFINE VARIABLE vLocalField AS CHARACTER NO-UNDO.
  DEFINE VARIABLE vi          AS INTEGER   NO-UNDO.
  DEFINE VARIABLE vNumfields  AS INTEGER   NO-UNDO.
  DEFINE VARIABLE hBuf        AS HANDLE    NO-UNDO.
    
  hBuf = hTT:DEFAULT-BUFFER-HANDLE.    
  vNumFields = hBuf:NUM-FIELDS.
  fieldLoop:
  DO vi = 1 TO vNumFields:
    IF hBuf:BUFFER-FIELD (vi):SERIALIZE-NAME = SFDCField THEN
    DO:
      vLocalField = hBuf:BUFFER-FIELD (vi):NAME.
      LEAVE fieldLoop. 
    END.      
  END.
    
  RETURN vLocalField.
 
END FUNCTION.
