/* --------------------------------------------------------------------
  Replication Temp Tables
  
  The temp table and temp field names match your local schema. The
  serialize-name attribute matches the SFDC table and field names.
 ---------------------------------------------------------------------- */


DEFINE TEMP-TABLE ttCustomer NO-UNDO SERIALIZE-NAME "Account"
  FIELD Id                AS CHARACTER 
  FIELD CustNum           AS CHARACTER SERIALIZE-NAME "AccountNumber" INITIAL ?
  FIELD cust-name         AS CHARACTER SERIALIZE-NAME "Name"
  FIELD Street            AS CHARACTER SERIALIZE-NAME "BillingStreet"
  FIELD City              AS CHARACTER SERIALIZE-NAME "BillingCity"
  FIELD State             AS CHARACTER SERIALIZE-NAME "BillingState"
  FIELD Country           AS CHARACTER SERIALIZE-NAME "BillingCountry"
  FIELD PostalCode        AS CHARACTER SERIALIZE-NAME "BillingPostalCode"
  FIELD OwnerId           AS CHARACTER 
  FIELD NumberOfEmployees AS INTEGER INITIAL ?      
  INDEX id id.    
      
DEFINE TEMP-TABLE ttContact NO-UNDO SERIALIZE-NAME "Contact"
  FIELD Id           AS CHARACTER 
  FIELD ctc-no       AS INTEGER   SERIALIZE-NAME "SkipSFDC" INITIAL ?   
  FIELD fname        AS CHARACTER SERIALIZE-NAME "FirstName"
  FIELD lname        AS CHARACTER SERIALIZE-NAME "LastName"
  FIELD email-addr   AS CHARACTER SERIALIZE-NAME "Email"
  FIELD fax          AS CHARACTER SERIALIZE-NAME "Fax"
  FIELD Street       AS CHARACTER SERIALIZE-NAME "MailingStreet"
  FIELD City         AS CHARACTER SERIALIZE-NAME "MailingCity"
  FIELD state        AS CHARACTER SERIALIZE-NAME "MailingState"
  FIELD postalCode   AS CHARACTER SERIALIZE-NAME "MailingPostalcode"
  FIELD Country      AS CHARACTER SERIALIZE-NAME "MailingCountry"
  FIELD phone        AS CHARACTER SERIALIZE-NAME "Phone"
  FIELD salutation   AS CHARACTER SERIALIZE-NAME "Salutation"
  FIELD ctc-title    AS CHARACTER SERIALIZE-NAME "Title"
  FIELD AccountId    AS CHARACTER SERIALIZE-NAME "AccountId"
  FIELD department   AS CHARACTER 
  INDEX id id .       

DEFINE TEMP-TABLE ttProduct NO-UNDO SERIALIZE-NAME "Product2"
  FIELD id        AS CHARACTER
  FIELD prod_num  AS INTEGER   SERIALIZE-NAME "Orion_Id__c" INITIAL ?      
  FIELD class     AS CHARACTER SERIALIZE-NAME "Family"
  FIELD prod-code AS CHARACTER SERIALIZE-NAME "ProductCode"
  FIELD prod-desc AS CHARACTER SERIALIZE-NAME "Name"
  FIELD comments  AS CHARACTER EXTENT 5 SERIALIZE-NAME "Description"
  FIELD IsActive  AS LOGICAL   SERIALIZE-NAME "IsActive" INIT TRUE 
  INDEX id id .

DEFINE TEMP-TABLE ttOpportunity NO-UNDO 
  FIELD id                AS CHARACTER 
  FIELD oppName           AS CHARACTER SERIALIZE-NAME "Name"
  FIELD CloseDate         AS DATE      SERIALIZE-NAME "CloseDate"
  FIELD StageName         AS CHARACTER SERIALIZE-NAME "StageName"
  FIELD cust-no           AS INTEGER   SERIALIZE-NAME "Customer_Number__c" INITIAL ?
  FIELD employee_no       AS INTEGER   SERIALIZE-NAME "Employee_no__c" INITIAL ?
  FIELD opportunity_no    AS INTEGER   SERIALIZE-NAME "Opportunity_no__c" INITIAL ?
  FIELD yearmonth         AS CHARACTER SERIALIZE-NAME "YearMonth__c" INITIAL ?
  FIELD oppType           AS CHARACTER SERIALIZE-NAME "Type"
  FIELD OwnerId           AS CHARACTER 
  FIELD AccountId         AS CHARACTER 
  FIELD RecordTypeId      AS CHARACTER 
  FIELD Sales_Status__c   AS CHARACTER 
  FIELD Estimated_Date__c AS DATE 
  INDEX id id.

  
DEFINE TEMP-TABLE ttOpportunityLineItem NO-UNDO
  FIELD id                     AS CHARACTER
  FIELD currency               AS CHARACTER SERIALIZE-NAME "Currency__c"
  FIELD dist-code              AS CHARACTER SERIALIZE-NAME "Dist_code__c"
  FIELD lineStatus             AS CHARACTER SERIALIZE-NAME "LineStatus__c"
  FIELD listPrice              AS DECIMAL   SERIALIZE-NAME "ListPrice__c"
  FIELD opportunityLineItem_no AS INTEGER   SERIALIZE-NAME "OpportunityLineItem_Number__c" INITIAL ?
  FIELD opportunity_no         AS INTEGER   SERIALIZE-NAME "Opportunity_number__c" INITIAL ?
  FIELD ord-no                 AS INTEGER   SERIALIZE-NAME "Order_number__c" INITIAL ?
  FIELD parentClass            AS CHARACTER SERIALIZE-NAME "Parent_Class__c"
  FIELD periodSeries           AS INTEGER   SERIALIZE-NAME "PeriodSeries__c" INITIAL ?
  FIELD Potential              AS DECIMAL   SERIALIZE-NAME "Potential__c" INITIAL ?
  FIELD UnitPrice              AS DECIMAL   SERIALIZE-NAME "UnitPrice" INITIAL ?
  FIELD prod_num               AS INTEGER   SERIALIZE-NAME "Prod_num__c" INITIAL ?
  FIELD renewedAmount          AS DECIMAL   SERIALIZE-NAME "Renewed_Amount__c" INITIAL ?
  FIELD site-no                AS INTEGER   SERIALIZE-NAME "Site_no__c" INITIAL ?
  FIELD Quantity               AS INTEGER   SERIALIZE-NAME "Quantity" INITIAL ?
  FIELD PricebookEntryId       AS CHARACTER SERIALIZE-NAME "PricebookEntryId"
  FIELD OpportunityId          AS CHARACTER SERIALIZE-NAME "OpportunityId" 
  FIELD isRetired              AS LOGICAL   SERIALIZE-NAME "Is_Retired__c"
  INDEX id id.

DEFINE TEMP-TABLE ttPricebookEntry NO-UNDO /* PricebookEntry */
  FIELD id               AS CHARACTER 
  FIELD IsActive         AS LOGICAL   SERIALIZE-NAME "IsActive" INIT TRUE
  FIELD UnitPrice        AS INTEGER   SERIALIZE-NAME "UnitPrice"  INITIAL ?
  FIELD Pricebook2Id     AS CHARACTER 
  FIELD Product2Id       AS CHARACTER
  FIELD UseStandardPrice AS LOGICAL   INIT FALSE  
  FIELD ProductCode      AS CHARACTER SERIALIZE-NAME "skipSFDC"
  FIELD prod-desc        AS CHARACTER SERIALIZE-NAME "skipSFDC"
  . 

DEFINE TEMP-TABLE ttAccountTeamMember NO-UNDO 
  FIELD id             AS CHARACTER
  FIELD cust-no        AS INTEGER   SERIALIZE-NAME "skipSFDC" INITIAL ?
  FIELD AccountId      AS CHARACTER SERIALIZE-NAME "AccountId"
  FIELD TeamMemberRole AS CHARACTER SERIALIZE-NAME "TeamMemberRole"
  FIELD SFDCUserId     AS CHARACTER SERIALIZE-NAME "UserId"
  INDEX id id.
     
/* --------------------------------------------------- */    
/* Support temp-tables and tables indirectly repicated */
/* --------------------------------------------------- */
  
DEFINE TEMP-TABLE ttOpportunityContactRole NO-UNDO
  FIELD ContactId     AS CHARACTER 
  FIELD OpportunityId AS CHARACTER .
  
DEFINE TEMP-TABLE ttLead NO-UNDO 
  FIELD id                      AS CHARACTER 
  FIELD LeadSource              AS CHARACTER 
  FIELD ctc-adr-city            AS CHARACTER SERIALIZE-NAME "City"
  FIELD ctc-adr-ctry            AS CHARACTER SERIALIZE-NAME "Country"
  FIELD ctc-adr-st              AS CHARACTER SERIALIZE-NAME "State"
  FIELD street                  AS CHARACTER SERIALIZE-NAME "Street"
  FIELD ctc-adr-zip             AS CHARACTER SERIALIZE-NAME "PostalCode"          
  FIELD ctc-title               AS CHARACTER SERIALIZE-NAME "Title"        
  FIELD cust-name               AS CHARACTER SERIALIZE-NAME "Company"   
  FIELD fname                   AS CHARACTER SERIALIZE-NAME "FirstName"      
  FIELD lname                   AS CHARACTER SERIALIZE-NAME "LastName"        
  FIELD market                  AS CHARACTER SERIALIZE-NAME "Market__c"      
  FIELD phone                   AS CHARACTER SERIALIZE-NAME "Phone"        
  FIELD salu                    AS CHARACTER SERIALIZE-NAME "Salutation"       
  FIELD AreaOfInterest          AS CHARACTER SERIALIZE-NAME "Area_of_Interest__c"  
  FIELD ExistingCustomer        AS CHARACTER SERIALIZE-NAME "Are_You_an_Existing_Customer__c"  
  FIELD RoleInSpendingDecisions AS CHARACTER SERIALIZE-NAME "Role_In_Spending_Decisions__c"
  FIELD WhereDoYouWork          AS CHARACTER SERIALIZE-NAME "Where_Do_You_Work__c" 
  FIELD email-addr              AS CHARACTER SERIALIZE-NAME "Email"
  FIELD jobRole                 AS CHARACTER SERIALIZE-NAME "Job_Role__c"
  FIELD RecordTypeId            AS CHARACTER   
  FIELD EmailOptIn              AS LOGICAL   SERIALIZE-NAME "HasOptedOutOfEmail"
  FIELD MarketInterest          AS CHARACTER SERIALIZE-NAME "Marketing_Interest__c"
  INDEX id id.
    
DEFINE TEMP-TABLE ttStaticResource NO-UNDO
  FIELD id            AS CHARACTER 
  FIELD srName        AS CHARACTER SERIALIZE-NAME "Name"
  FIELD srDescription AS CHARACTER SERIALIZE-NAME "Description"
  FIELD contentType   AS CHARACTER . 

DEFINE TEMP-TABLE ttUser NO-UNDO
  FIELD id             AS CHARACTER 
  FIELD Email          AS CHARACTER 
  FIELD FirstName      AS CHARACTER 
  FIELD LastName       AS CHARACTER 
  FIELD NAME           AS CHARACTER 
  FIELD EmployeeNumber AS INTEGER INITIAL ?
  FIELD isActive       AS LOGICAL 
  INDEX id             id
  INDEX employeeNumber employeeNumber.


DEFINE TEMP-TABLE ttId NO-UNDO
  FIELD id     AS CHARACTER
  FIELD fromId AS CHARACTER .
    
DEFINE TEMP-TABLE ttFieldsToNull NO-UNDO
  FIELD id            AS CHARACTER 
  FIELD fieldName     AS CHARACTER 
  INDEX id id.    

DEFINE TEMP-TABLE ttUserMapping NO-UNDO
  FIELD SFDCUserId  AS CHARACTER
  FIELD employee_no AS INTEGER INITIAL ?
  FIELD isActive    AS LOGICAL 
  INDEX employee_no employee_no.
  
DEFINE TEMP-TABLE ttCurrencyType NO-UNDO 
  FIELD id             AS CHARACTER 
  FIELD ConversionRate AS DECIMAL INITIAL ?
  FIELD IsoCode        AS CHARACTER 
  FIELD IsActive       AS LOGICAL 
  INDEX isocode isocode.
  
